import SwiftUI

struct Connection:View {
    
    @Environment(\.dismiss) var dismiss
    @EnvironmentObject var tabList:Tabs
    @Binding var directoryJson:Data
    @Binding var display:String
    @State var message1:String? = nil
    @State var message2:String? = nil
    @State var message1Sent = false
    @State var message2Sent = false
    @State var buttonMessage1 = "Paste message 1"
    @State var buttonMessage2 = "Paste message 2"
    @State var otherMessage1:String? = nil
    @State var otherMessage2:String? = nil
    @State var isConfirming = false
    @State var cipher = Cipher()
    
    var body:some View {
            VStack {
                Button("Reset",action:reset)
                
                if case .initial = cipher.state {
                    Button("Generate message 1",action:makemessage1)
                }
                if let m = message1 {
                    ShareBox(label:"Message 1:",toShare:m,done:$message1Sent)
                }
                if case .keycreated = cipher.state {
                    HStack {
                        Button(buttonMessage1,action:pastemessage1)
                        Button("OK",action:domessage1)
                            .disabled(!message1Sent || otherMessage1 == nil)
                    }
                }
                if case .secretbuilt = cipher.state {
                    Button("Generate message 2",action:makemessage2)
                }
                if let m = message2 {
                    ShareBox(label:"Message 2:",toShare:m,done:$message2Sent)
                }
                if case .signaturesent = cipher.state {
                    HStack {
                        Button(buttonMessage2,action:pastemessage2)
                        Button("OK",action:domessage2)
                            .disabled(!message2Sent || otherMessage2 == nil)
                    }
                }
            }
            .buttonStyle(.bordered)
            .font(.system(size:20))
            Spacer()
        
    }
    
    
    
    func reset() {
        cipher.reset()
        display = "---"
        message1 = nil
        message2 = nil
        buttonMessage1 = "Paste message 1"
        buttonMessage2 = "Paste message 2"
        message1Sent = false
        message2Sent = false
    }
    
    func makemessage1() {
        do {
            message1 = try cipher.message1()
        }
        catch {
            sherr(error)
        }
    }
    
    func makemessage2() {
        do {
            message2 = try cipher.message2()
        }
        catch {
            sherr(error)
        }
    }
    
    func shortString(_ m:String) -> String {
        if (m.count<15) {
            return m
        } else {
            let index = m.index(m.startIndex, offsetBy: 15)
            return m[..<index]+"..."
        }
    }
    
    func pastemessage1() {
        guard let m = UIPasteboard.general.string else {
            display = "No string to paste"
            return
        }
        otherMessage1 = m
        buttonMessage1 = shortString(m)
    }
    
    func domessage1() {
        if message1 == otherMessage1 {
            display = "This is your message 1"
            return
        }
        defer {
            buttonMessage1 = "Paste message 1"
            message1 = nil
            otherMessage1 = nil
            message1Sent = false
        }
        do {
            let d = try cipher.handle(message1:otherMessage1!)
            let formatter = DateComponentsFormatter()
            formatter.unitsStyle = .full
            formatter.allowedUnits = [.minute,.hour,.day]
            display = formatter.string(from:d,to:Date()) ?? DateFormatter().string(from: d)
        }
        catch {
            sherr(error)
        }
    }
    
    func copymessage2() {
        do {
            UIPasteboard.general.string = try cipher.message2()
        }
        catch {
            sherr(error)
        }
    }
    
    func pastemessage2() {
        guard let m = UIPasteboard.general.string else {
            display = "No string to paste"
            return
        }
        otherMessage2 = m
        buttonMessage2 = shortString(m)
    }
    
    func domessage2() {
        if message2 == otherMessage2 {
            display = "This is your message 2"
            return
        }
        defer {
            buttonMessage2 = "Paste message 2"
            message2 = nil
            otherMessage2 = nil
            message2Sent = false
        }
        do {
            let (d,key) = try cipher.handle(message2:otherMessage2!,directory: directory())
            let index = tabList.list.count
            tabList.list.append(EncTab(display:$display,name:d,index:index,key:key))
            dismiss()
        }
        catch {
            sherr(error)
        }
    }
    
    func sherr(_ error:Error) {
        display = strerr(error)
    }
    
    func directory() -> [DirEntry] {
        do {
            return try JSONDecoder().decode([DirEntry].self, from: directoryJson)
        }
        catch {
            sherr(error)
            directoryJson="[]".data(using:.ascii)!
            return []
        }
    }
    
}

struct Connection_Previews: PreviewProvider {
    @AppStorage("directory",store:.standard) static var directoryJson:Data = "[]".data(using: .ascii)!
    @State static var display = "---"
    static var previews: some View {
        VStack {
        Text(display)
        Connection(directoryJson:$directoryJson,display:$display)
        }
    }
}
