import SwiftUI
import CryptoKit

struct DirEntry:Codable,Identifiable {
    var id:String
    var pem:String
}

class Tabs:ObservableObject {
    @Published var list:[EncTab]=[]
}

struct ContentView: View {
    let columns = [GridItem](repeating: .init(.flexible(minimum: 0, maximum: .infinity)), count: 3)
    @State var display = "---"
    @State var isConfirming = false
    @AppStorage("directory",store:.standard) var directoryJson:Data = "[]".data(using: .ascii)!
    @StateObject var tabList = Tabs()
    
    //var directory:[DirEntry] 
    
    var body: some View {
        VStack{
            Text(display)
                .fixedSize(horizontal: false, vertical: true)
            
        NavigationView {
                
                VStack(spacing:20) {
                    HStack {
                        NavigationLink {
                            VStack {
                                Button("Delete key",action: {isConfirming  = true})
                                    .confirmationDialog(
                                        "Are you sure you want to delete the current key?",
                                        isPresented: $isConfirming
                                    )  { 
                                        Button("Confirm",action:{doDelKey()})
                                        Button("Cancel", role: .cancel, action:{isConfirming = false})
                                    }
                                ShareBox(label:"Public key:",toShare:pubkey())
                            }
                            .navigationTitle("Identity")
                        } label: {
                            Text("Manage identity")
                        }
                        
                        DirectoryEditor(directoryJson:$directoryJson,display:$display)
                        
                    }
                    ScrollView {
                    LazyVGrid(columns: columns,
                              alignment:.leading) {
                        NavigationLink {
                            Connection (directoryJson:$directoryJson,display:$display)
                                .navigationTitle("New")
                        } label: {
                            /*VStack {
                                Image(systemName: "person.fill.badge.plus")
                                Spacer()
                            }*/
                            IconText(name:"person.fill.badge.plus",label:"New..")
                        }
                        ForEach(tabList.list) {tab in tab}
                    }
                    }
                }
                .navigationTitle("E2EE")
                .navigationBarTitleDisplayMode(.inline)
                
        }
        .navigationViewStyle(.stack)
        .buttonStyle(.bordered)
        .font(.system(size:20))
        .environmentObject(tabList)
        }
    }
    
    //
    
    func sherr(_ error:Error) {
        display = strerr(error)
    }
    
    func doDelKey() {
        do {
            try delKey(label)
            display = "Key successfully deleted"
        }
        catch {
            sherr(error)
        }
    }
    
    // directory editor
    
    func directory() -> [DirEntry] {
        do {
            return try JSONDecoder().decode([DirEntry].self, from: directoryJson)
        }
        catch {
            sherr(error)
            directoryJson="[]".data(using:.ascii)!
            return []
        }
    }
    
    func setDirectory(_ dir:[DirEntry]) {
        do {
            directoryJson = try JSONEncoder().encode(dir)
        }
        catch {
            sherr(error)
        }
    } 
    /*
     @State var newname = ""
     @State var newpem = ""
     @State var addpemmessage = "New name and PEM signature"
     */
}
