import SwiftUI

struct DirectoryAdd:View {
    @Environment(\.dismiss) var dismiss
    @Binding var directoryJson:Data
    @Binding var display:String
    @State var editdir = false
    @State var newname = ""
    @State var newpem = """
-----BEGIN PUBLIC KEY-----
-----END PUBLIC KEY-----
"""
    @State var addpemmessage = "New name and PEM signature"
    var body:some View {
        VStack(spacing:10) {
            Section {
                Text(addpemmessage)
            }
            VStack {
                HStack {
                    TextField("Name",text:$newname)
                    Button("Paste",action:pastepem)
                }
                TextEditor(text:$newpem)
                    .border(Color.black,width:1)
                    .frame(minHeight:200)
                Button("Add",action:adddirentry)
            }
        }
        .navigationTitle("Add...")
    }
    func adddirentry() {
        if (newname == "") {
            addpemmessage = "Empty name"
            return
        }
        var dir = directory()
        if dir.contains(where:{e in e.id == newname}) {
            addpemmessage = "Name already used"
            return
        }
        guard let _ = try? PubKey(pemRepresentation:newpem) else {
            addpemmessage = "Invalid signature"
            return
        }
        dir.append(DirEntry(id:newname,pem:newpem))
        setDirectory(dir)
        addpemmessage = "Signature successful imported"
        newpem = """
-----BEGIN PUBLIC KEY-----
-----END PUBLIC KEY-----
"""
        newname = ""
        dismiss()
    }
    
    func pastepem() {
        guard let m = UIPasteboard.general.string else {
            display = "No string to paste"
            return
        }
        newpem = m
    }
    
    func directory() -> [DirEntry] {
        do {
            return try JSONDecoder().decode([DirEntry].self, from: directoryJson)
        }
        catch {
            sherr(error)
            directoryJson="[]".data(using:.ascii)!
            return []
        }
    }
    
    func setDirectory(_ dir:[DirEntry]) {
        do {
            directoryJson = try JSONEncoder().encode(dir)
        }
        catch {
            sherr(error)
        }
    } 
    
    func sherr(_ error:Error) {
        display = strerr(error)
    }
}

struct DirectoryEditor:View {
    @Binding var directoryJson:Data
    @Binding var display:String

    var body:some View {
        NavigationLink {
            List {
                Section {
                    NavigationLink {
                        DirectoryAdd(directoryJson:$directoryJson,display:$display)
                    } label: {
                        Text("Add...")
                    }
                }
                Section {
                    ForEach(directory()) {entry in
                        Text(entry.id)
                    }
                    .onDelete(perform:delete)
                }
            }
            .navigationTitle("Directory")
        } label: {
            Text("Edit Directory")
        }
    }
    
    func directory() -> [DirEntry] {
        do {
            return try JSONDecoder().decode([DirEntry].self, from: directoryJson)
        }
        catch {
            sherr(error)
            directoryJson="[]".data(using:.ascii)!
            return []
        }
    }
    
    func setDirectory(_ dir:[DirEntry]) {
        do {
            directoryJson = try JSONEncoder().encode(dir)
        }
        catch {
            sherr(error)
        }
    } 
    /*
     @State var newname = ""
     @State var newpem = ""
     @State var addpemmessage = "New name and PEM signature"
     */
    
    func delete(pos:IndexSet) {
        var dir = directory()
        dir.remove(atOffsets:pos)
        setDirectory(dir)
    }
    
    func sherr(_ error:Error) {
        display = strerr(error)
    }
}

struct Directory_Previews: PreviewProvider {
    @AppStorage("directory",store:.standard) static var directoryJson:Data = "[]".data(using: .ascii)!
    @State static var display = "---"
    static var previews: some View {
        VStack {
            Text(display)
            NavigationView {
                DirectoryEditor(directoryJson:$directoryJson,display:$display) 
            }
        }
        VStack {
            Text(display)
                DirectoryAdd(directoryJson:$directoryJson,display:$display) 
        }
    }
}

