import SwiftUI
import CryptoKit

struct EncTab:View,Identifiable {
    @EnvironmentObject var tabList:Tabs
    @State var sharearea = false
    @State var area = ""
    @Binding var display:String
    let id:String = UUID().uuidString
    let name:String
    let index:Int
    let key:SymmetricKey
    var body:some View {
        NavigationLink {
            
                
                TextEditor(text:$area)
                    .border(Color.black,width:1)
                    .frame(minHeight:200)
                    .toolbar {
                        ToolbarItemGroup(placement: .navigationBarLeading) {
                        Button(action:clear) {
                            Image(systemName: "trash")
                        }
                        Button(action:encrypt) {
                            Image(systemName: "lock")
                        }
                        Button(action:copy) {
                            Image(systemName: "doc.on.doc")
                        }
                        Button(action: {sharearea = true}){
                            Image(systemName: "square.and.arrow.up")
                        }
                        .sheet(isPresented: $sharearea) {
                            ShareSheet(str: $area)
                        }
                        Button(action:decrypt) {
                            Image(systemName: "lock.open")
                        }
                        Button(action:paste) {
                            Image(systemName: "doc.on.clipboard")
                        }
                        }
                        ToolbarItemGroup(placement: .navigationBarTrailing) {
                        Button(action: close) {
                            Image(systemName: "clear")
                        }
                        }
                    }
            .buttonStyle(.plain)
        } label: {
            IconText(name:"person.fill",label:name)
        }
    }
    
    func close() {
        tabList.list.remove(at:index)
    }
    
    func clear() {
        area = ""
    }
    
    func paste() {
        guard let m = UIPasteboard.general.string else {
            display = "No string to paste"
            return
        }
        area = m
    }
    
    func copy() {
        UIPasteboard.general.string = area
    }
    
    func encrypt() {
        do {
            area = try cipher(message:area)
            display = "Encryption successful"
        }
        catch {
            sherr(error)
        }
    }
    
    func decrypt() {
        do {
            area = try decipher(message:area)
            display = "Decryption successful"
        }
        catch {
            sherr(error)
        }
    }
    
    func cipher(message:String) throws -> String {
        let data = message.data(using:.utf8)!
        return try Cipher.cipher(data,key)
    }
    
    func decipher(message:String) throws -> String {
        let dec = try Cipher.decipher(message,key)
        return String(bytes:dec,encoding: .utf8)!
    }
    
    func sherr(_ error:Error) {
        switch (error) {
        case let err as CryptoError :
            display = err.message
        default :
            display = error.localizedDescription
        }
    }
}

struct EncTab_Previews: PreviewProvider {
    @State static var display = "---"
    static var previews: some View {
        VStack {
            Text(display)
            NavigationView {
                EncTab(display:$display,name:"Test",index:0,key:SymmetricKey(size:.bits256))
            }
        }
    }
}
