import SwiftUI
import CoreImage.CIFilterBuiltins

struct QR: View {
    @Binding var source:String
    var body: some View {
        VStack {
            HStack(alignment: .bottom) {
            Text("Scan the QR code with the photo app and click on the bottom right icon to copy the text to the clipboard:")
                Image(systemName: "qrcode.viewfinder")
            }
        if let image = qrcode(source) {
            Image(uiImage:image)
                .interpolation(.none)
                .resizable()
                .scaledToFit()
                
        }
        else {
            Image(systemName: "clear.fill")
                .symbolRenderingMode(.multicolor)
        }
        }
    }
}

struct QR_Previews: PreviewProvider {
    @State static var source = "QR Test"
    static var previews: some View {
            QR(source:$source)
    }
}

func qrcode(_ label: String) -> UIImage? {
    let filter = CIFilter.qrCodeGenerator()
    filter.message = label.data(using: String.Encoding.ascii)!
    if let ans = filter.outputImage {
        if let img = CIContext().createCGImage(ans,from:ans.extent) {
            return UIImage(cgImage: img)
        }
    }
    
    return nil
}
