import SwiftUI

struct ShareBox:View {
    let label:String
    @State var toShare:String
    var done:Binding<Bool>? = nil
    @State var share = false
    @State var qr = false
    var body: some View {
        HStack {
            Text(label)
            Button(action:doPaste) {
                Image(systemName: "doc.on.doc")
            }
            Button(action: {share = true; setDone()}) {
                Image(systemName: "square.and.arrow.up")
            }
            Button(action: {qr = true; setDone()}) {
                Image(systemName: "qrcode")
            }
            .sheet(isPresented: $share) {
                ShareSheet(str: $toShare)
            }
            .sheet(isPresented: $qr) {
                QR(source:$toShare)
            }
        }
    }
    
    func doPaste() {
        UIPasteboard.general.string = toShare
        setDone()
    }
    
    func setDone() {
        if let d = done {
            d.wrappedValue = true
        }
    }
}

struct ShareSheet: UIViewControllerRepresentable {
    
    @Binding var str:String
    
    func makeUIViewController(context: Context) -> UIActivityViewController {
        return UIActivityViewController(
            activityItems: [str],
            applicationActivities: nil)
    }
    
    func updateUIViewController(_ uiViewController: UIActivityViewController, context: Context) {
        
    }
}
