import CryptoKit
import Security
import Foundation

typealias PrivKey = SecureEnclave.P256.Signing.PrivateKey
typealias PubKey = P256.Signing.PublicKey
typealias Sig = P256.Signing.ECDSASignature

let label = "mykey"
func delKey(_ account: String) throws {
    let query = [kSecClass: kSecClassGenericPassword,
           kSecAttrAccount: account,
kSecUseDataProtectionKeychain: true,
            kSecReturnData: true] as [String: Any]
    
    SecItemDelete(query as CFDictionary)
}

func storeKey(_ key: PrivKey,_ account: String) throws {
    let query =     [kSecClass: kSecClassGenericPassword,
               kSecAttrAccount: account,
            kSecAttrAccessible: kSecAttrAccessibleWhenUnlockedThisDeviceOnly,
 kSecUseDataProtectionKeychain: true,
             kSecValueData: key.dataRepresentation] as [String: Any]
    
    // Add the key data.
    let status = SecItemAdd(query as CFDictionary, nil)
    guard status == errSecSuccess else {
        throw CryptoError("Unable to store item",status)
    }
}

func getKey(_ account: String) throws -> PrivKey? {
    let query = [kSecClass: kSecClassGenericPassword,
       kSecAttrAccount: account,
kSecUseDataProtectionKeychain: true,
        kSecReturnData: true] as [String: Any]

    //SecItemDelete(query as CFDictionary)
    //return nil;

    var item: CFTypeRef?
    switch SecItemCopyMatching(query as CFDictionary, &item) {
    case errSecSuccess:
        guard let data = item as? Data else { return nil }
        //print(data.base64EncodedString())
        return try PrivKey(dataRepresentation: data)  // Convert back to a key.
    case errSecItemNotFound: return nil
    case let status: throw CryptoError("Keychain read failed",status)
    }
}

func key() throws -> PrivKey {
    if let key = try getKey(label) {
        return key
    }
    //let secValue = SecAccessControlCreateFlags.init(rawValue: SecAccessControlCreateFlags.privateKeyUsage.rawValue | SecAccessControlCreateFlags.userPresence.rawValue)
    
    let acc = SecAccessControlCreateWithFlags(nil, kSecAttrAccessibleWhenPasscodeSetThisDeviceOnly,[.privateKeyUsage,.userPresence], nil)
    let key = try PrivKey(compactRepresentable: true, accessControl: acc!,authenticationContext:nil)
    //print(key.dataRepresentation.base64EncodedString())
    try storeKey(key,label)
    return key
}

func sign(_ data:MyData) throws -> Sig {
    let key = try key()
    //print(Data(data).base64EncodedString())
    let hash = MyDigest.hash(data:data)
    //print(hash)
    let sig = try key.signature(for:hash)
    //print(try key.publicKey.isValidSignature(sig, for: hash))
    //print(Data(sig.derRepresentation).base64EncodedString())
    //print(Data(sig.rawRepresentation).base64EncodedString())
    //print(key.publicKey.pemRepresentation)
    return sig
}

func verif(_ data:MyData, _ signature:Sig, _ key:P256.Signing.PublicKey) -> Bool {
    let hash = MyDigest.hash(data:data)
    return key.isValidSignature(signature, for: hash)
}

func getSigner(data:MyData,sig:Sig, directory:[DirEntry]) -> String? {
    for entry in directory {
        if let pub = try? PubKey(pemRepresentation: entry.pem) {
            if verif(data, sig, pub) {
                return entry.id;
            }
        }
    }
    return nil
}

func pubkey() -> String {
    do {
        return try key().publicKey.pemRepresentation
    }
    catch {
        return "\(error)"
    }
}
