import SwiftUI
import CryptoKit

class CryptoError:Error {
    var message:String
    var localizedDescription:String {
        get {
            return message
        }
    }
    init(_ message:String) {
        self.message = message
    }
    init(_ info:String, _ status:OSStatus) {
        let message = SecCopyErrorMessageString(status, nil).debugDescription
        self.message=info+", "+message
    }
}

/*func derlen(_ d:Data) -> Int? {
    if d.count < 2 {
        return nil
    }
    let l = d[1]
    if l&0x80 == 0 {
        return Int(l)+2
    }
    let n = Int(l&0x7f)
    if d.count < n+2 {
        return nil
    }
    var r = 0
    for i in 2..<n+2 {
        r = (r<<8)+Int(d[i])
    }
    return r+n+2
}*/

func createRandom(count:Int) throws -> MyData {
    //[UInt8](repeating:0,count:MyDigest.byteCount)
    //print("---- \(bytes.count)")
    var bytes = [UInt8](repeating:0,count:count)
    let status = SecRandomCopyBytes(kSecRandomDefault, count, &bytes)
    guard status == errSecSuccess else {
        throw CryptoError("Cannot create random salt",status)
    }
    return bytes
    //print("-- - \(bytes.count)")
}

func trimString(_ str:inout String) {
    str.removeAll(where:{c in c.isWhitespace})
}

// a of size 8
func timeBigEndian() -> MyData {
    var a = [UInt8](repeating:0,count:8)
    let x = UInt64(Date().timeIntervalSince1970)
    for i in 0..<8 {
        a[7-i] = UInt8((x >> (8*i))&0xff)
    }
    return a
}

func bigendianToDate(_ a:MyData) throws -> Date {
    if (a.count < 8) {
        throw CryptoError("Invalid data size")
    }
    var x = UInt64.zero
    for i in 0..<8 {
        x = x|(UInt64(a[7-i])<<(8*i))
    }
    return Date(timeIntervalSince1970: Double(x))
}

func strerr(_ error:Error) -> String {
    switch (error) {
    case let err as CryptoError :
        return err.message
    default :
        return error.localizedDescription
    }
}
